#################################################################################################################
### R code for Simulation Studies----Section 5.1.2 
### Power Comparison of the GMCTs with the Likelihood Ratio Test Refinement of MCP-Mod, Table 3
### MA and MCDERMOTT Generalized Multiple Contrast Tests in Dose-Response Studies
### Same simulation setting as Section 3.3 Gutjahr and Bornkamp (2017) except a smaller number of repetitions (10000)
##################################################################################################################

### Simulation studies were conducted using R studio version: 3.5.0 under Nemo, a Red Hat Linux server in the
### Department of Biostatistics and Computational Biology at University of Rochester Medical Center. 
### Random numbers were generated using R package 'mvtnorm' version 1.0-11.
### Please note that different versions of R studio might generate slightly different simulation results since the
### R package 'mvtnorm' might have different version and thus have different random number generators even for the
### same seed. But the difference should be negligible (within simulation error). 

#R code of the likelihood ratio (LR) test (Gutjahr and Bornkamp 2017) are obtained from the Supplementary Materials
#at the Biometrics website on Wiley Online Library.

library(chebsample)
library(mnll)
library(mvtnorm)
library(DoseFinding)

############## Preparation ##############
set.seed(12345)
BLOCK_SIZE=100
MAX_BLOCKS=100
nSim=100

## generate data ##
d=c(0,0.05, 0.2, 0.6, 1)
dose=rep(d, each=20)

################# LR ############################
getB=function(n)
  t(svd(rep(1, n), n , n)$u[,-1])
B=getB(100)

## simulate standardized predictions emax model ("normal" interval)
bndsEmax <- c(0.001,1.5)
o <- list(x=dose, lambda = bndsEmax[1], upsilon = bndsEmax[2])
q <- cheb_q(o, g_emax, 1e-6, 1e-6, 1e-6, 1e-6, 1e-6)
emax <- function(x, gamma) x / (x + gamma)
tx_gamma <- function(x, gamma) stat(B, emax(x, gamma))
sims_Emax <- sapply(esample(o, q, nSim), function(gamma) tx_gamma(x=o$x, gamma))

## exponential model
bndsExpo=c(0.1, 2)
o=list(x=dose, lambda = bndsExpo[1], upsilon = bndsExpo[2])
q=cheb_q(o, g_exponential, 1e-6, 1e-6, 1e-6, 1e-6, 1e-6)
expo=function(x, gamma) exp(x / gamma)-1
tx_gamma=function(x, gamma) stat(B, expo(x, gamma))
sims_Expo=sapply(esample(o, q, nSim), function(gamma) tx_gamma(x=o$x, gamma))

## linear model (nothing to simulate, just one shape)
sims_Lin=stat(B, o$x)

#critical value of the LR test
sims_EmaxLinExpo <- cbind(sims_Emax, sims_Lin, sims_Expo)
r=uniroot(function(x){
  mnll_pvalue(x, sims_EmaxLinExpo, block_size = BLOCK_SIZE,
              max_blocks = MAX_BLOCKS) - 0.05
}, c(0.01, 0.5))

#the critical value r=0.2097797

#####################################################################
########## generate data                                    #########
#####################################################################
#Constant
mu0=rep(0.2,5)
#Linear
mu1=d 
#Emax
mu2=d/(0.2+d) 
#Exp 1
mu3=exp(d/0.1)-1 
#Exp 2
mu4=exp(2*d*log(6))-1 
#Sigmoid Emax
mu5=d^4/(d^4+0.05^4) 

#non-centrality paramters
delta1=1.65629
delta2=2.503821

################# MCP-Mod ############################
#Calculate contrasts
contrast=function(mu){
  (mu-mean(mu))/norm(mu-mean(mu),type='2')
}

c1=contrast(mu1) #Linear
c2=contrast(mu2) #Emax
c3=contrast(mu3) #Exp 1
c4=contrast(mu4) #Exp 2

rho12=sum(c1*c2)/sqrt(sum(c1^2)*sum(c2^2))
rho13=sum(c1*c3)/sqrt(sum(c1^2)*sum(c3^2))
rho14=sum(c1*c4)/sqrt(sum(c1^2)*sum(c4^2))
rho23=sum(c2*c3)/sqrt(sum(c2^2)*sum(c3^2))
rho24=sum(c2*c4)/sqrt(sum(c2^2)*sum(c4^2))
rho34=sum(c3*c4)/sqrt(sum(c3^2)*sum(c4^2))

Sigma=matrix(c(1,rho12, rho13,rho14,
               rho12, 1, rho23, rho24,
               rho13, rho23, 1, rho34,
               rho14, rho24, rho34, 1),nrow=4,ncol=4)

PoC_LR=function(mu=mu0,delta=delta1){
  
set.seed(12345)
nsim=10000
x=rep(mu, each=20)
sigma=sqrt(sum((B%*%x)^2))/delta
#sigma=1  #Type I error

reject_LR=rep(NA,nsim)
pvalue_MCPMod=rep(NA,nsim)
pvalue_F=rep(NA,nsim)
pvalue_N=rep(NA,nsim)

for (s in 1:nsim){
  #Generate data with 5 dosage group, each group has 20 patients
  Y=rmvnorm(20,mean=mu,sigma = diag(sigma^2,nrow=5,ncol=5))
  resp=c(Y)
  
################################## LR test ##################################
  ## fit models and calculate test statistics
  mods=vector("list", 3)
  mods[[1]]=fitMod(dose,resp, model="emax", bnds=bndsEmax)
  mods[[2]]=fitMod(dose,resp, model="linear")
  mods[[3]]=fitMod(dose,resp, model="exponential",bnds = bndsExpo)
  names(mods)=lapply(mods, function(x) attr(x, "model"))
  
  ## function to calculate test statistic
  getTstat <- function(mod, y)
  {
    cf <- coef(mod)
    pred <- (predict(mod, predType = "l")-cf[1])/cf[2]
    B <- getB(length(pred))
    pr <- stat(B,pred)
    dat <- stat(B,y)
    sum(pr*dat)
  }
  tt <- lapply(mods, getTstat, y=resp)  #test-statistic
  R=max(c(tt$emax,tt$linear,tt$exponential))
  
  #reject H0 if
  reject_LR[s]=(R>r)

################################## MCP-Mod ##################################   
  
  #Multiple contrast test
  ybar1=mean(resp[dose==0])
  ybar2=mean(resp[dose==0.05])
  ybar3=mean(resp[dose==0.2])
  ybar4=mean(resp[dose==0.6])
  ybar5=mean(resp[dose==1])
  ybar=c(ybar1,ybar2,ybar3,ybar4,ybar5)
  
  #Contrast estimate of 6 models
  ce1=sum(c1*ybar)
  ce2=sum(c2*ybar)  
  ce3=sum(c3*ybar)
  ce4=sum(c4*ybar)
  ce=c(ce1,ce2,ce3,ce4)
  
  ddf=20*5-5  
  #T-value
  S2=(sum((resp[dose==0]-ybar1)^2)+sum((resp[dose==0.05]-ybar2)^2)+
        sum((resp[dose==0.2]-ybar3)^2)+sum((resp[dose==0.6]-ybar4)^2)+
        sum((resp[dose==1]-ybar5)^2))/ddf
  
  t1=ce1/sqrt(S2*sum(c1^2)/20)
  t2=ce2/sqrt(S2*sum(c2^2)/20)
  t3=ce3/sqrt(S2*sum(c3^2)/20)
  t4=ce4/sqrt(S2*sum(c4^2)/20)
  t=c(t1,t2,t3,t4)
  
  #Dependent p values
  p1=1-pt(t1,df=ddf)
  p2=1-pt(t2,df=ddf)
  p3=1-pt(t3,df=ddf)
  p4=1-pt(t4,df=ddf)
  
  #PoC p-value of MCP-Mod
  pvalue_MCPMod[s]=1-pmvt(lower=rep(-Inf,4),upper =rep(max(t),4), corr=Sigma, df=ddf)[1]

################################## GMCTs ##################################
  
#################Fisher's Combination test###############
  cov12_F=3.279*rho12+0.711*rho12^2+0.727/ddf+0.129*rho12/ddf-0.768*rho12^2/ddf
  cov13_F=3.279*rho13+0.711*rho13^2+0.727/ddf+0.129*rho13/ddf-0.768*rho13^2/ddf
  cov14_F=3.279*rho14+0.711*rho14^2+0.727/ddf+0.129*rho14/ddf-0.768*rho14^2/ddf
  cov23_F=3.279*rho23+0.711*rho23^2+0.727/ddf+0.129*rho23/ddf-0.768*rho23^2/ddf
  cov24_F=3.279*rho24+0.711*rho24^2+0.727/ddf+0.129*rho24/ddf-0.768*rho24^2/ddf
  cov34_F=3.279*rho34+0.711*rho34^2+0.727/ddf+0.129*rho34/ddf-0.768*rho34^2/ddf
  
  #The mean and variance of the Fisher's statistic 
  E=2*4
  var=4*4+2*(cov12_F+cov13_F+cov14_F+cov23_F+cov24_F+cov34_F)
  #Degrees of freedom f  
  f=2*E^2/var
  c=var/(2*E)
  
  phiFobs=-2*(log(p1)+log(p2)+log(p3)+log(p4))
  pvalue_F[s] = 1-pchisq(phiFobs/c,df=f)  
  
########Inverse Normal Combination method########
  cov12_N=rho12+0.00019*rho12^3-0.02378*rho12/ddf+0.01891*rho12^3/ddf
  cov13_N=rho13+0.00019*rho13^3-0.02378*rho13/ddf+0.01891*rho13^3/ddf
  cov14_N=rho14+0.00019*rho14^3-0.02378*rho14/ddf+0.01891*rho14^3/ddf
  cov23_N=rho23+0.00019*rho23^3-0.02378*rho23/ddf+0.01891*rho23^3/ddf
  cov24_N=rho24+0.00019*rho24^3-0.02378*rho24/ddf+0.01891*rho24^3/ddf
  cov34_N=rho34+0.00019*rho34^3-0.02378*rho34/ddf+0.01891*rho34^3/ddf
 
  pvalue_N[s]=1-pnorm(1/sqrt(4)*(qnorm(1-p1)+qnorm(1-p2)+qnorm(1-p3)+
                                    qnorm(1-p4)),mean=0,
                       sd=sqrt(1+2/4*(cov12_N+cov13_N+cov14_N+cov23_N+cov24_N+cov34_N)))  
}

PoC_prob=c(mean(reject_LR),
mean(pvalue_MCPMod<=0.05),
mean(pvalue_F<=0.05),
mean(pvalue_N<=0.05))

return(PoC_prob)

}

#Type I error (sigma=1) of LR, MCP-Mod, Fisher and Inverse normal 
#PoC_LR(mu=mu0,delta=delta1) 
#0.0487 0.0491 0.0492 0.0486

start_time <- Sys.time()
PoC_LR(mu=mu1,delta=delta1) #0.4364 0.4731 0.4936 0.4948
PoC_LR(mu=mu2,delta=delta1) #0.4288 0.4320 0.4087 0.4052
PoC_LR(mu=mu3,delta=delta1) #0.3946 0.4427 0.4479 0.4429
PoC_LR(mu=mu4,delta=delta1) #0.4135 0.4539 0.4808 0.4775
PoC_LR(mu=mu5,delta=delta1) #0.4145 0.3629 0.2996 0.2932

PoC_LR(mu=mu1,delta=delta2) #0.7397 0.7708 0.7911 0.7914
PoC_LR(mu=mu2,delta=delta2) #0.7316 0.7431 0.7078 0.6958
PoC_LR(mu=mu3,delta=delta2) #0.7050 0.7478 0.7459 0.7364
PoC_LR(mu=mu4,delta=delta2) #0.7265 0.7656 0.7855 0.7842
PoC_LR(mu=mu5,delta=delta2) #0.7163 0.6549 0.5329 0.5061
end_time <- Sys.time()
# Running time 49.78656 min
end_time - start_time

